/**
 * Copyright 2006-8 Simon Andrews
 *
 *    This file is part of ChIPMonk.
 *
 *    ChIPMonk is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    ChIPMonk is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with ChIPMonk; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package uk.ac.bbsrc.babraham.ChipMonk.Analysis.Statistics;

import uk.ac.bbsrc.babraham.ChipMonk.ChipMonkException;

public class TwoSampleTTest {

	public static double calculatePValue (double [] values1, double [] values2) throws ChipMonkException {

		if (values1 == null  || values2 == null) {
			throw new ChipMonkException ("Lists of values for t-test cannot be null");
		}
		if (values1.length < 2 || values2.length < 2) {
			throw new ChipMonkException("At least two values are required in each set for a t-test");
		}

		double mean1 = 0;
		for (int i=0;i<values1.length;i++){
			mean1 += values1[i];
		}
		mean1 /= values1.length;
		
		double var1 = 0;
		for (int i=0;i<values1.length;i++) {
			var1 += Math.pow((values1[i]-mean1),2);
		}
		var1 /= (values1.length-1);

		double mean2 = 0;
		for (int i=0;i<values2.length;i++){
			mean2 += values2[i];
		}
		mean2 /= values2.length;
		
		double var2 = 0;
		for (int i=0;i<values2.length;i++) {
			var2 += Math.pow((values2[i]-mean2),2);
		}
		var2 /= (values2.length-1);

		double tValue = ((mean1-mean2)/Math.sqrt((var1/values1.length)+(var2/values2.length)));
				
		if (tValue < 0)
			tValue = 0-tValue;

		double pValue = TDistribution.tProb((values1.length+values2.length)-2,tValue);

		return pValue;
	}
	
}
