package uk.ac.bbsrc.babraham.ChipMonk.DataParsers;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.util.Enumeration;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.filechooser.FileFilter;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableCellRenderer;

import uk.ac.bbsrc.babraham.ChipMonk.ChipMonkException;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.Chromosome;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.Genome;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.Probe;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.ProbeSet;
import uk.ac.bbsrc.babraham.ChipMonk.Preferences.ChipMonkPreferences;

public class GenericProbeParser extends JDialog implements ActionListener{
	
	private GenericParser parser;

	private JComboBox delimiters;
	private JComboBox startRow;
	private JComboBox probeCol;
	private JComboBox chrCol;
	private JComboBox startCol;
	private JComboBox endCol;
	private JComboBox midCol;
	private JButton importButton;
	private String [] previewData = new String [50];
	private ProbeTableModel model;
	private JTable table;
	private JScrollPane tablePane = null;
	private File probeFile;
	private Vector listeners = new Vector();
	
	public GenericProbeParser (GenericParser parser) {
		super(parser.application(),"Import Probe Data");
		this.parser = parser;
	}
	
	public void addDataParserListener (DataParserListener l) {
		listeners.add(l);
	}
	
	public void removeDataParserListener (DataParserListener l) {
		listeners.remove(l);
	}
	
	public void parseProbes () {
		
		JFileChooser chooser = new JFileChooser(ChipMonkPreferences.getInstance().getDataLocation());
		chooser.setMultiSelectionEnabled(false);
		chooser.setFileFilter(new FileFilter() {
		
			public String getDescription() {
				return "Text Files";
			}
		
			public boolean accept(File f) {
				if (f.isDirectory() || f.getName().toLowerCase().endsWith(".txt")) {
					return true;
				}
				else {
					return false;
				}
			}
		
		});
		
		int result = chooser.showOpenDialog(this);
		if (result == JFileChooser.CANCEL_OPTION) {
			this.dispose();
			return;
		}

		probeFile = chooser.getSelectedFile();

		// We can now read the first 50 lines from the file
		try {
			BufferedReader br = new BufferedReader(new FileReader(probeFile));
			int x=0;
			String line;
			while (true) {
				line = br.readLine();
				// We could (theoretically) have less than 50 lines of text
				if (line == null) line = "";
				if (x>49) break;
				previewData[x] = new String(line);
				x++;
			}
			br.close();
		}
		catch (Exception e) {
			e.printStackTrace();
		}
		

		getContentPane().setLayout(new BorderLayout());
		
		JPanel optionPanel = new JPanel();
		
		optionPanel.setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.gridx=0;
		c.gridy=0;
		c.weightx=0.5;
		c.weighty=0.5;
		c.fill = GridBagConstraints.NONE;
		
		optionPanel.add(new JLabel("Column Delimiter"),c);
		delimiters = new JComboBox(new String [] {"Tab","Space","Comma"});
		delimiters.addActionListener(this);
		c.gridx=1;
		c.weightx=0.1;
		optionPanel.add(delimiters,c);
		
		c.gridx=0;
		c.weightx=0.5;
		c.gridy++;
		optionPanel.add(new JLabel("Start at Row"),c);
		
		String [] rowList = new String[51];
		for (int i=0;i<51;i++) {
			rowList[i] = ""+i;
		}
		
		startRow = new JComboBox(rowList);
		startRow.addActionListener(this);
		c.gridx=1;
		c.weightx=0.1;
		optionPanel.add(startRow,c);
		
		c.gridx=0;
		c.weightx=0.5;
		c.gridy++;
		optionPanel.add(new JLabel("Probe ID Col"),c);
		probeCol = new JComboBox();
		probeCol.addActionListener(this);
		c.gridx=1;
		c.weightx=0.1;
		optionPanel.add(probeCol,c);
		
		c.gridx=0;
		c.weightx=0.5;
		c.gridy++;
		optionPanel.add(new JLabel("Chr Col"),c);
		chrCol = new JComboBox();
		chrCol.addActionListener(this);
		c.gridx=1;
		c.weightx=0.1;
		optionPanel.add(chrCol,c);
		
		c.gridx=0;
		c.weightx=0.5;
		c.gridy++;
		optionPanel.add(new JLabel("Start Col"),c);
		startCol = new JComboBox();
		startCol.addActionListener(this);
		c.gridx=1;
		c.weightx=0.1;
		optionPanel.add(startCol,c);
		
		c.gridx=0;
		c.weightx=0.5;
		c.gridy++;
		optionPanel.add(new JLabel("End Col"),c);
		endCol = new JComboBox();
		endCol.addActionListener(this);
		c.gridx=1;
		c.weightx=0.1;
		optionPanel.add(endCol,c);

		c.gridx=0;
		c.weightx=0.5;
		c.gridy++;
		optionPanel.add(new JLabel("Position Col"),c);
		midCol = new JComboBox();
		midCol.addActionListener(this);
		c.gridx=1;
		c.weightx=0.1;
		optionPanel.add(midCol,c);

		c.gridx=0;
		c.weightx=0.5;
		c.gridy++;
		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridwidth=2;
		importButton = new JButton("Import");
		importButton.setActionCommand("import");
		importButton.setEnabled(false);
		importButton.addActionListener(this);
		optionPanel.add(importButton,c);
		
		getContentPane().add(optionPanel,BorderLayout.EAST);
		model = new ProbeTableModel();
		
		updateTable();

		setSize(800,600);
		setLocationRelativeTo(parser);
		setVisible(true);
	}
	
	private void updateTable () {
		
		// We need to rebuild the list of column numbers.
		String [] columnList = new String [model.getColumnCount()];
		columnList[0] = null;
		for (int i=1;i<model.getColumnCount();i++) {
			columnList[i] = ""+i;
		}
		chrCol.removeAllItems();
		startCol.removeAllItems();
		endCol.removeAllItems();
		midCol.removeAllItems();
		probeCol.removeAllItems();

		for (int i=0;i<columnList.length;i++) {
			chrCol.addItem(columnList[i]);
			startCol.addItem(columnList[i]);
			endCol.addItem(columnList[i]);
			midCol.addItem(columnList[i]);
			probeCol.addItem(columnList[i]);
		}
		chrCol.validate();
		startCol.validate();
		endCol.validate();
		midCol.validate();
		probeCol.validate();


		
		
		if (tablePane != null) {
			model.fireTableStructureChanged();
			model.fireTableDataChanged();
		}
		else {
			table = new JTable(model);
			TableCellRenderer r = new MyTableCellRenderer();
			table.setDefaultRenderer(Object.class, r);
			tablePane = new JScrollPane(table);
			getContentPane().add(tablePane,BorderLayout.CENTER);
			getContentPane().validate();
		}
	}

	private String getDelimiter () {
		if (((String)(delimiters.getSelectedItem())).equals("Tab")) {
			return "\t";
		}
		if (((String)(delimiters.getSelectedItem())).equals("Space")) {
			return " ";
		}
		if (((String)(delimiters.getSelectedItem())).equals("Comma")) {
			return ",";
		}
		return null;
	}
	
	private void checkImportOK () {
		// See if we can enable the import button
		if (startRow.getSelectedItem()==null) {
			importButton.setEnabled(false);
			return;
		}
		
		// We need a chr col to be set
		if (chrCol.getSelectedItem() == null) {
			importButton.setEnabled(false);
			return;
		}

		// We need a probe name col to be set
		if (probeCol.getSelectedItem() == null) {
			importButton.setEnabled(false);
			return;
		}

		// We either need a mid value or both a start and an end value
		if ((midCol.getSelectedItem()!=null) || (startCol.getSelectedItem()!=null && endCol.getSelectedItem()!=null)){
			importButton.setEnabled(true);
			return;
		}
		else {
			importButton.setEnabled(false);
		}
	}
	

	public void actionPerformed(ActionEvent ae) {
		
		if (ae.getSource() == delimiters) {
			updateTable();
			return;
		}
		
		if (ae.getSource() == startRow) {
			model.fireTableDataChanged();
			return;
		}
		
		if (ae.getSource()!=importButton) {
			checkImportOK();
			return;
		}
		
		// If we get here then we're doing the actual import
		
		try {
			
			// Get the values for the things we need
			String delimetersValue = getDelimiter();
			int startRowValue = Integer.parseInt((String)startRow.getSelectedItem());
			int probeColValue = Integer.parseInt((String)probeCol.getSelectedItem())-1;
			int chrColValue = Integer.parseInt((String)chrCol.getSelectedItem())-1;
			int startColValue = -1;
			int endColValue = -1;
			int midColValue = -1;
			
			boolean useMid = false;
			
			if (midCol.getSelectedItem()!=null) {
				useMid = true;
				midColValue = Integer.parseInt((String)midCol.getSelectedItem())-1;
			}
			else {
				startColValue = Integer.parseInt((String)startCol.getSelectedItem())-1;
				endColValue = Integer.parseInt((String)endCol.getSelectedItem())-1;
			}
			
			BufferedReader br  = new BufferedReader(new FileReader(probeFile));
			String line;
			// First skip the header lines
			for (int i=0;i<startRowValue;i++) {
				line = br.readLine();
				if (line == null) {
					throw new Exception ("Ran out of file before skipping all of the header lines");
				}
			}
			
			int maxIndexValue = 0;
			if (probeColValue > maxIndexValue) maxIndexValue = probeColValue;
			if (chrColValue > maxIndexValue) maxIndexValue = chrColValue;
			if (startColValue > maxIndexValue) maxIndexValue = startColValue;
			if (endColValue > maxIndexValue) maxIndexValue = endColValue;
			if (midColValue > maxIndexValue) maxIndexValue = midColValue;
			
			
			ProbeSet p = parser.application().dataCollection().probeSet();
			Genome g = parser.application().genome();
			
			// Now process the rest of the file
			while ((line = br.readLine())!= null) {
				String [] sections = line.split(delimetersValue);
				
				// Check to see if we've got enough data to work with
				if (maxIndexValue >= sections.length) {
					// Try to at least get the probeID
					if (probeColValue >= sections.length) {
						sendWarning(new ChipMonkException("Not enough data to get a probe name on line '"+line+"'"));
						continue; // Skip this line...						
					}

					// We'll settle for adding a probe with no position
					
					// See if we actually have a name...
					if (sections[probeColValue].length()==0) {
						sendWarning(new ChipMonkException("No probe name found on short line '"+line+"'"));
						continue;
					}
					
					// See if this probe already exists
					if (p.containsProbeWithName(sections[probeColValue])) {
						sendWarning(new ChipMonkException("Duplicate probe name "+sections[probeColValue]));
					}

					// We can now make the new probe
					Probe newProbe = new Probe(new String(sections[probeColValue]),null,0);
					p.addProbe(newProbe);
					
					continue;

				}
				
				// For the chromosome section remove chr from the front of any
				// name
				if (sections[chrColValue].toLowerCase().startsWith("chr")) {
					sections[chrColValue] = sections[chrColValue].replaceFirst("\\s*[Cc][Hh][Rr]\\s*","");
//					System.out.println("After removal chromosome was "+sections[chrColValue]);
				}
				
				// Check that the probe name isn't blank (not acceptable here)
				if (sections[probeColValue].length() == 0) {
					sendWarning(new ChipMonkException("Blank probe name found in line '"+line+"'"));
					continue;
				}
				
				// See if this probe already exists
				if (p.containsProbeWithName(sections[probeColValue])) {
					sendWarning(new ChipMonkException("Duplicate probe name "+sections[probeColValue]));
					continue;
				}

				int mid;
				try {
					if (useMid) {
						mid = Integer.parseInt(sections[midColValue]);
					}
					else {
						int start = Integer.parseInt(sections[startColValue]);
						int end = Integer.parseInt(sections[endColValue]);
						
						if (end > start) {
							mid = start+((end-start)/2);
						}
						else {
							mid = end+((start-end)/2);
						}
					}
				}
				catch (NumberFormatException e) {
					sendWarning(new ChipMonkException("Location of "+sections[probeColValue]+" was not an integer"));
					continue;
				}
				
				Chromosome c = g.getChromosome(sections[chrColValue]);
				if (c == null) {
					sendWarning(new ChipMonkException("Probe "+sections[probeColValue]+" could not be mapped to chromosome '"+sections[chrColValue]+"'"));
				}
				else if (mid > c.length()) {
					sendWarning(new ChipMonkException("Probe "+sections[2]+" has a position "+mid+" beyond the end of chr"+c.name()+" ("+c.length()+")"));
					c = null;
					mid = 0;
				}

				// We can now make the new probe
				Probe newProbe = new Probe(new String(sections[probeColValue]),c,mid);
				p.addProbe(newProbe);
//				System.out.println("Added probe "+newProbe.name()+" on "+newProbe.chromosome()+" at pos "+newProbe.position());
			}
			
			// We're finished with the file.
			br.close();
			
			if (p.getAllProbes().length == 0) {
				throw new Exception("No probes imported");
			}
			JOptionPane.showMessageDialog(this,"Imported "+p.getAllProbes().length+" probes","Import Succesful",JOptionPane.INFORMATION_MESSAGE);
		}
		catch (Exception ex) {
			Enumeration e = listeners.elements();
			while (e.hasMoreElements()) {
				((DataParserListener)e.nextElement()).dataExceptionReceived(ex);
			}
			ex.printStackTrace();
			JOptionPane.showMessageDialog(this,"Error importing probe data: "+ex.getMessage(),"Error",JOptionPane.ERROR_MESSAGE);
			return;
		}

		setVisible(false);

		Enumeration e = listeners.elements();
		while (e.hasMoreElements()) {
			((DataParserListener)e.nextElement()).dataProcessingFinished(false);
		}
		parser.probesImported();
		dispose();
	}
	
	private void sendWarning (Exception ex) {
		Enumeration e = listeners.elements();
		while (e.hasMoreElements()) {
			((DataParserListener)e.nextElement()).dataWarningReceived(ex);
		}
		
	}

	private class ProbeTableModel extends AbstractTableModel {

		public int getRowCount() {
			int startRowValue = 1;
			if (startRow.getSelectedItem()!=null) {
				startRowValue = Integer.parseInt((String)startRow.getSelectedItem());
			}
			
			if (startRowValue < 1 || startRowValue > 50) {
				startRowValue = 1;
			}
			
			return 51-startRowValue;
		}
		
		public String getColumnName (int column) {
			if (column==0) {
				return "Row number";
			}
			else {
				return "Col "+column;
			}
		}

		public int getColumnCount() {
			int max = 1;
			for (int i=0;i<previewData.length;i++) {
				String [] sections = previewData[i].split("["+getDelimiter()+"]");
				if (sections.length > max) {
					max = sections.length;
				}
			}
			return max+1;
		}
		


		public Object getValueAt(int r, int c) {
					
			if (c==0) {
				return ""+r;
			}
			
			c-=1;
			
			int startRowValue = 1;
//			if (startRow.getText().length() > 0) {
//				startRowValue = Integer.parseInt(startRow.getText());
//			}
//			
//			if (startRowValue < 1 || startRowValue > 50) {
//				startRowValue = 1;
//			}


			String [] sections = previewData[r+(startRowValue-1)].split("["+getDelimiter()+"]");
			if (sections.length <=c) {
				return null;
			}
			else {
				return sections[c];
			}
		}
	}
	
	private class MyTableCellRenderer extends DefaultTableCellRenderer {
		
		// The only point of this class is so we can tell which rows we are ignoring.
		// We do this by selcting the ignored ones so they go grey.
		
		public Component getTableCellRendererComponent (JTable table,Object value,boolean isSelected, boolean hasFocus, int r, int c) {

			if (r<Integer.parseInt((String)startRow.getSelectedItem())) {
				isSelected = true;
			}
			else {
				isSelected = false;
			}
			Component cell = super.getTableCellRendererComponent(table, value, isSelected, hasFocus, r, c);
			
			return cell;
		}
				
	}
}
