/**
 * Copyright 2006-8 Simon Andrews
 *
 *    This file is part of ChIPMonk.
 *
 *    ChIPMonk is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    ChIPMonk is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with ChIPMonk; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package uk.ac.bbsrc.babraham.ChipMonk.DataTypes;

import java.util.Arrays;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import uk.ac.bbsrc.babraham.ChipMonk.ChipMonkException;

public class Chromosome implements Comparable {

	private String name;
	private int length = 0;
	private Hashtable features = new Hashtable();
	
	public Chromosome (String name) {
		this.name = new String(name);
	}
	
	public String name () {
		return name;
	}
	
	public int length () {
		return length;
	}
	
	public void setLength (int length) throws ChipMonkException {
		
		if (length > this.length) {
			this.length = length;
		}
	}
	
	public void addFeature (Feature f) {
		if (features.containsKey(f.type())) {
			((Vector)(features.get(f.type()))).add(f);
		}
		else {
			Vector v = new Vector();
			v.add(f);
			features.put(f.type(),v);
		}
	}
	
	public String [] listAvailableFeatureTypes () {
		String [] types = new String [features.size()];
		
		Enumeration e = features.keys();
		int c=0;
		while (e.hasMoreElements()) {
			types[c] = (String)e.nextElement();
			c++;
		}
		Arrays.sort(types);
		return types;
	}
	
	public Feature [] getFeaturesForType (String type) {
		if (features.containsKey(type)) {
			Vector v = (Vector)features.get(type);
			Feature [] f = new Feature[v.size()];
			
			int c=0;
			Enumeration e = v.elements();
			while (e.hasMoreElements()) {
				f[c] = (Feature)e.nextElement();
				c++;
			}
			return f;
		}
		
		else {
			return new Feature [0];
		}
	}

	public String toString () {
		return name;
	}
	
	public int compareTo(Object o) {
		try {
			return (new Integer(Integer.parseInt(name)).compareTo(new Integer(Integer.parseInt(((Chromosome)o).name()))));
		}
		catch (Exception e) {
			return name.compareTo(((Chromosome)o).name());
		}
	}
	
}
