/**
 * Copyright 2006-8 Simon Andrews
 *
 *    This file is part of ChIPMonk.
 *
 *    ChIPMonk is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    ChIPMonk is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with ChIPMonk; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package uk.ac.bbsrc.babraham.ChipMonk.Dialogs;

import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import uk.ac.bbsrc.babraham.ChipMonk.ChipMonkApplication;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.DataGroup;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.DataSet;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.DataStore;
import uk.ac.bbsrc.babraham.ChipMonk.Dialogs.Renderers.TypeColourRenderer;

public class DataTrackSelector extends JDialog implements ActionListener, ListSelectionListener {
	
	private ChipMonkApplication application;
	private DefaultListModel availableGroupModel = new DefaultListModel();
	private DefaultListModel availableSetModel = new DefaultListModel();
	private JList availableGroupList;
	private JList availableSetList;
	private DefaultListModel usedModel = new DefaultListModel();
	private JList usedList;
	private JButton addButton;
	private JButton removeButton;
	private JButton upButton;
	private JButton downButton;
	private TypeColourRenderer renderer = new TypeColourRenderer();
	
	public DataTrackSelector (ChipMonkApplication application) {
		super(application,"Select Data Tracks");
		setSize(400,300);
		setLocationRelativeTo(application);
		
		this.application = application;
		
		getContentPane().setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.insets = new Insets(2,2,2,2);
		c.gridx=0;
		c.gridy=0;
		c.weightx=0.9;
		c.weighty=0.9;
		c.fill = GridBagConstraints.BOTH;
		
		JPanel availablePanel = new JPanel();
		availablePanel.setLayout(new GridBagLayout());
		GridBagConstraints c2 = new GridBagConstraints();
		c2.gridx=0;
		c2.gridy=0;
		c2.weightx=1;
		c2.weighty=0.01;
		c2.fill = GridBagConstraints.HORIZONTAL;
		availablePanel.add(new JLabel("Available Tracks",JLabel.CENTER),c2);
		
		c2.gridy++;
		availablePanel.add(new JLabel("Data Groups",JLabel.LEFT),c2);
		
		c2.gridy++;
		c2.weighty=1;
		c2.fill=GridBagConstraints.BOTH;
		
		availableGroupList = new JList(availableGroupModel);
		availableGroupList.addListSelectionListener(this);
		availableGroupList.setCellRenderer(renderer);
		availablePanel.add(new JScrollPane(availableGroupList),c2);

		c2.gridy++;
		c2.weighty=0.01;
		c2.fill=GridBagConstraints.HORIZONTAL;
		availablePanel.add(new JLabel("Data Sets",JLabel.LEFT),c2);

		c2.gridy++;
		c2.weighty=1;
		c2.fill=GridBagConstraints.BOTH;

		availableSetList = new JList(availableSetModel);
		availableSetList.addListSelectionListener(this);
		availableSetList.setCellRenderer(renderer);
		availablePanel.add(new JScrollPane(availableSetList),c2);
		
		
		getContentPane().add(availablePanel,c);

		c.gridx++;
		c.weightx = 0.2;
		c.fill = GridBagConstraints.NONE;
		
		JPanel middlePanel = new JPanel();
		middlePanel.setLayout(new GridLayout(4,1));
		addButton = new JButton("Add");
		addButton.setActionCommand("add");
		addButton.addActionListener(this);
		addButton.setEnabled(false);
		middlePanel.add(addButton);
		
		removeButton = new JButton("Remove");
		removeButton.setActionCommand("remove");
		removeButton.addActionListener(this);
		removeButton.setEnabled(false);
		middlePanel.add(removeButton);
		
		upButton = new JButton("Move up");
		upButton.setActionCommand("up");
		upButton.addActionListener(this);
		upButton.setEnabled(false);
		middlePanel.add(upButton);
		
		downButton = new JButton("Move down");
		downButton.setActionCommand("down");
		downButton.addActionListener(this);
		downButton.setEnabled(false);
		middlePanel.add(downButton);
		
		getContentPane().add(middlePanel,c);
		
		c.gridx++;
		c.weightx = 0.9;
		c.fill = GridBagConstraints.BOTH;

		JPanel usedPanel = new JPanel();
		usedPanel.setLayout(new BorderLayout());
		usedPanel.add(new JLabel("Displayed Tracks",JLabel.CENTER),BorderLayout.NORTH);
		usedList = new JList(usedModel);
		usedList.addListSelectionListener(this);
		usedList.setCellRenderer(renderer);
		usedPanel.add(new JScrollPane(usedList),BorderLayout.CENTER);
		getContentPane().add(usedPanel,c);
		
		c.gridx=0;
		c.gridy++;
		c.gridwidth=3;
		c.weighty = 0.01;
		c.fill = GridBagConstraints.NONE;

		JPanel bottomPanel = new JPanel();
		JButton cancelButton = new JButton("Cancel");
		cancelButton.setActionCommand("cancel");
		cancelButton.addActionListener(this);
		bottomPanel.add(cancelButton);
		
		JButton okButton = new JButton("OK");
		okButton.setActionCommand("ok");
		okButton.addActionListener(this);
		getRootPane().setDefaultButton(okButton);
		bottomPanel.add(okButton);
		
		getContentPane().add(bottomPanel,c);
		
		DataSet [] availableSets = application.dataCollection().getAllDataSets();
		for (int i=0;i<availableSets.length;i++) {
			availableSetModel.addElement(availableSets[i]);
		}
		
		DataGroup [] availableGroups = application.dataCollection().getAllDataGroups();
		for (int i=0;i<availableGroups.length;i++) {
			availableGroupModel.addElement(availableGroups[i]);
		}
		
		DataStore [] drawnStores = application.drawnDataStores();
		for (int i=0;i<drawnStores.length;i++) {
			if (drawnStores[i] instanceof DataSet) {
				availableSetModel.removeElement(drawnStores[i]);
			}
			else {
				availableGroupModel.removeElement(drawnStores[i]);				
			}
			usedModel.addElement(drawnStores[i]);
		}
		
		setVisible(true);
		
		// A bif of a hack to make sure all lists
		// resize as they should...
		availableSetModel.addElement("temp");
		availableGroupModel.addElement("temp");
		usedModel.addElement("temp");
		validate();
		availableSetModel.removeElement("temp");
		availableGroupModel.removeElement("temp");
		usedModel.removeElement("temp");
	}

	public void actionPerformed(ActionEvent ae) {
		String c = ae.getActionCommand();

		if (c.equals("add")) {
			Object [] adds = availableGroupList.getSelectedValues();
			for (int i=0;i<adds.length;i++) {
				usedModel.addElement(adds[i]);
				availableGroupModel.removeElement(adds[i]);
			}
			adds = availableSetList.getSelectedValues();
			for (int i=0;i<adds.length;i++) {
				usedModel.addElement(adds[i]);
				availableSetModel.removeElement(adds[i]);
			}
		}
		else if (c.equals("remove")) {
			Object [] removes = usedList.getSelectedValues();
			for (int i=0;i<removes.length;i++) {
				usedModel.removeElement(removes[i]);
				if (removes[i] instanceof DataSet) {
					availableSetModel.addElement(removes[i]);
				}
				else {
					availableGroupModel.addElement(removes[i]);
				}
			}			
		}
		else if (c.equals("up")) {
			int s = usedList.getSelectedIndex();
			Object current = usedModel.elementAt(s);
			Object above = usedModel.elementAt(s-1);
			usedModel.setElementAt(current,s-1);
			usedModel.setElementAt(above,s);
			usedList.setSelectedIndex(s-1);
		}
		else if (c.equals("down")) {
			int s = usedList.getSelectedIndex();
			Object current = usedModel.elementAt(s);
			Object below = usedModel.elementAt(s+1);
			usedModel.setElementAt(current,s+1);
			usedModel.setElementAt(below,s);
			usedList.setSelectedIndex(s+1);			
		}
		else if (c.equals("cancel")) {
			setVisible(false);
			dispose();
		}
		else if (c.equals("ok")) {
			Object [] o = usedModel.toArray();
			DataStore [] s = new DataStore [o.length];
			for (int i=0;i<s.length;i++) {
				s[i] = (DataStore)o[i];
			}
			application.setDrawnDataStores(s);
			setVisible(false);
			dispose();
		}
		
	}

	public void valueChanged(ListSelectionEvent ae) {
		// Check to see if we can add anything...
		if (availableSetList.getSelectedIndices().length>0 || availableGroupList.getSelectedIndices().length >0) {
			addButton.setEnabled(true);
		}
		else {
			addButton.setEnabled(false);
		}
		
		//Or remove anything
		if (usedList.getSelectedIndices().length>0) {
			removeButton.setEnabled(true);
		}
		else {
			removeButton.setEnabled(false);
		}
		
		// If we can move up
		if (usedList.getSelectedIndices().length == 1 && usedList.getSelectedIndex() != 0) {
			upButton.setEnabled(true);
		}
		else {
			upButton.setEnabled(false);
		}
		
		// If we can move down
		if (usedList.getSelectedIndices().length == 1 && usedList.getSelectedIndex() != usedList.getLastVisibleIndex()) {
			downButton.setEnabled(true);
		}
		else {
			downButton.setEnabled(false);
		}
	}
}
