/**
 * Copyright 2006-8 Simon Andrews
 *
 *    This file is part of ChIPMonk.
 *
 *    ChIPMonk is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    ChIPMonk is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with ChIPMonk; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package uk.ac.bbsrc.babraham.ChipMonk.Dialogs.Filters;

import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import uk.ac.bbsrc.babraham.ChipMonk.ChipMonkApplication;
import uk.ac.bbsrc.babraham.ChipMonk.ChipMonkException;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.Probe;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.ProbeList;


public class CombineFilter extends JDialog implements ActionListener {

	private ChipMonkApplication application;
	private JComboBox firstList;
	private JComboBox type;
	private JComboBox secondList;
	private JButton okButton;

	public CombineFilter (ChipMonkApplication application) {
		super(application,"Combine Probe Lists...");
		setSize(400,200);
		setLocationRelativeTo(application);
		
		this.application = application;

		getContentPane().setLayout(new BorderLayout());
		
		JPanel choicePanel = new JPanel();
		choicePanel.setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.gridx=0;
		c.gridy=0;
		c.weightx=0.5;
		c.weighty=0.5;
		c.fill=GridBagConstraints.NONE;
		c.anchor=GridBagConstraints.CENTER;
		
		ProbeList [] userLists = application.dataCollection().probeSet().getAllProbeLists();
		
		if (userLists.length==0) {
			JOptionPane.showMessageDialog(application,"You don't have any probe lists to combine","Error",JOptionPane.ERROR_MESSAGE);
			dispose();
			return;
		}
		
		// We want to add an 'all probes' list
		Probe [] allProbes = application.dataCollection().probeSet().getAllProbes();
		ProbeList allProbesList = new ProbeList(application.dataCollection().probeSet(),"All Probes",null);
		for (int p=0; p<allProbes.length;p++) {
			allProbesList.addProbe(allProbes[p],null);
		}
		
		ProbeList [] lists = new ProbeList [userLists.length+1];
		lists[0] = allProbesList;
		for (int l=0;l<userLists.length;l++) {
			lists[l+1] = userLists[l];
		}
		
		firstList = new JComboBox(lists);
		choicePanel.add(firstList,c);
		
		c.gridy++;
		type = new JComboBox(new String [] {"AND","OR","BUTNOT"});
		choicePanel.add(type,c);
		
		c.gridy++;
		secondList = new JComboBox(lists);
		choicePanel.add(secondList,c);
		
		getContentPane().add(new JScrollPane(choicePanel),BorderLayout.CENTER);
		
		JPanel buttonPanel = new JPanel();
		JButton cancelButton = new JButton("Close");
		cancelButton.setActionCommand("cancel");
		cancelButton.addActionListener(this);
		buttonPanel.add(cancelButton);
		
		okButton = new JButton("Run Filter");
		okButton.setActionCommand("ok");
		okButton.addActionListener(this);
		buttonPanel.add(okButton);
		
		getContentPane().add(buttonPanel,BorderLayout.SOUTH);
		
		setVisible(true);
	}

	private void makeProbeList () {
		
		ProbeList listA = (ProbeList)firstList.getSelectedItem();
		String combine = (String)type.getSelectedItem();
		ProbeList listB = (ProbeList)secondList.getSelectedItem();
		
		if (listA == listB) {
			JOptionPane.showMessageDialog(this,"The same list was selected twice","Error",JOptionPane.ERROR_MESSAGE);
			return;			
		}
		
		ProbeList newList = new ProbeList(application.dataCollection().probeSet(),"","No Value");
		
		if (combine.equals("AND")) {
			combineAnd(listA,listB,newList);
		}
		else if (combine.equals("OR")) {
			combineOr(listA,listB,newList);
		}
		else if (combine.equals("BUTNOT")) {
			combineButnot(listA,listB,newList);
		}
		
		
		// See if any probes actually passed
		if (newList.getAllProbes().length == 0) {
			JOptionPane.showMessageDialog(this,"No probes were contained in the combined set","Info",JOptionPane.INFORMATION_MESSAGE);
			dispose();
			return;
		}
		
		// Ask for a name for the list
		String groupName=null;
		while (true) {
			groupName = (String)JOptionPane.showInputDialog(this,"Enter list name","Found "+newList.getAllProbes().length+" probes",JOptionPane.QUESTION_MESSAGE,null,null,listA.getListName()+" "+combine+" "+listB.getListName());
			if (groupName == null)
				return;  // They cancelled
				
				
			if (groupName.length() == 0)
				continue; // Try again
			
			break;
		}
		newList.setName(groupName);
		
		
		
		try {
			application.dataCollection().probeSet().addProbeList(newList);
		} catch (ChipMonkException e) {
			e.printStackTrace();
		}
//		setVisible(false);
//		dispose();
		
	}
	
	private void combineAnd (ProbeList a, ProbeList b, ProbeList n) {
		Probe [] p = a.getAllProbes();
		
		for (int i=0;i<p.length;i++) {
			if (b.containsProbe(p[i])) {
				n.addProbe(p[i],null);
			}
		}
	}
	

	private void combineOr (ProbeList a, ProbeList b, ProbeList n) {
		Probe [] p = a.getAllProbes();
		
		for (int i=0;i<p.length;i++) {
			n.addProbe(p[i],null);
		}
		
		p = b.getAllProbes();
		//Add only those not in a
		for (int i=0;i<p.length;i++) {
			if (! a.containsProbe(p[i]))
				n.addProbe(p[i],null);
		}
		
	}

	private void combineButnot (ProbeList a, ProbeList b, ProbeList n) {
		Probe [] p = a.getAllProbes();
		
		for (int i=0;i<p.length;i++) {
			if (! b.containsProbe(p[i])) {
				n.addProbe(p[i],null);
			}
		}
		
	}

	
	public void actionPerformed(ActionEvent ae) {
		if (ae.getActionCommand().equals("cancel")) {
			setVisible(false);
			dispose();
		}
		else if (ae.getActionCommand().equals("ok")) {
			makeProbeList();
		}
	}
	
}
