/**
 * Copyright 2006-8 Simon Andrews
 *
 *    This file is part of ChIPMonk.
 *
 *    ChIPMonk is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    ChIPMonk is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with ChIPMonk; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package uk.ac.bbsrc.babraham.ChipMonk.Dialogs;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileNotFoundException;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeSelectionModel;

import uk.ac.bbsrc.babraham.ChipMonk.ChipMonkApplication;
import uk.ac.bbsrc.babraham.ChipMonk.Preferences.ChipMonkPreferences;

public class GenomeSelector extends JDialog {
	
	private ChipMonkApplication application;
	
	private JTree tree;
	private JButton okButton;
	
	public GenomeSelector (ChipMonkApplication application) {
		super(application,"Select Genome...");
		this.application = application;
		setSize(300,250);
		setLocationRelativeTo(application);
		setModal(true);
		setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);

		getContentPane().setLayout(new BorderLayout());

		// Create the tree of available genomes
		DefaultMutableTreeNode root = new DefaultMutableTreeNode("Genomes");
		
		File[] genomes;
		try {
			genomes = ChipMonkPreferences.getInstance().getGenomeBase().listFiles();
		} 
		catch (FileNotFoundException e) {
			JOptionPane.showMessageDialog(application, "Couldn't find the folder containing your genomes.  Please check your file preferences", "Error getting genomes", JOptionPane.ERROR_MESSAGE);
			return;
		}
		
		for (int i=0;i<genomes.length;i++) {
			if (genomes[i].isDirectory() && genomes[i].canRead()) {
				DefaultMutableTreeNode genomeNode = new DefaultMutableTreeNode(genomes[i].getName());
				File [] assemblies = genomes[i].listFiles();
				if (assemblies == null) {
					continue;
				}
				for (int j=0;j<assemblies.length;j++) {
					if (assemblies[j].isDirectory()) {
						genomeNode.add(new AssemblyNode(assemblies[j]));
					}
				}
				root.add(genomeNode);
			}
		}

		DefaultTreeModel treeModel = new DefaultTreeModel(root);
		tree = new JTree(treeModel);
		tree.addTreeSelectionListener(new TreeListener());
		tree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);


		getContentPane().add(new JScrollPane(tree),BorderLayout.CENTER);
		
		
		
		// Create the buttons at the bottom.
		ButtonListener l = new ButtonListener();
		
		JPanel buttonPanel = new JPanel();
		
		JButton cancelButton = new JButton("Cancel");
		cancelButton.setActionCommand("cancel");
		cancelButton.addActionListener(l);
		buttonPanel.add(cancelButton);

		JButton importButton = new JButton("Import New");
		importButton.setActionCommand("import");
		importButton.addActionListener(l);
		buttonPanel.add(importButton);
		
		okButton = new JButton("OK");
		okButton.setActionCommand("ok");
		okButton.setEnabled(false);
		okButton.addActionListener(l);
		getRootPane().setDefaultButton(okButton);
		buttonPanel.add(okButton);
		
		getContentPane().add(buttonPanel,BorderLayout.SOUTH);
		
		
		setVisible(true);
	}

	private class ButtonListener implements ActionListener {

		public void actionPerformed(ActionEvent ae) {
			if (ae.getActionCommand().equals("ok")) {
				setVisible(false);
				
				// Remove any currently loaded data
				application.wipeAllData();
				
				// Now load the new genome.
				application.loadGenome(((AssemblyNode)tree.getSelectionPath().getLastPathComponent()).file());
				dispose();
			}
			else if (ae.getActionCommand().equals("import")) {
				try {
					new GenomeDownloadSelector(application);
				} catch (Exception e) {
					JOptionPane.showMessageDialog(application, "Error getting list of genomes", "Error", JOptionPane.ERROR_MESSAGE);
					e.printStackTrace();
				}
				setVisible(false);
				dispose();
			}
			else if (ae.getActionCommand().equals("cancel")) {
				setVisible(false);
				dispose();
			}
			
		}
	}
	
	private class TreeListener implements TreeSelectionListener {

		public void valueChanged(TreeSelectionEvent tse) {
			if (tree.getSelectionPath() != null && tree.getSelectionPath().getLastPathComponent() instanceof AssemblyNode) {
				okButton.setEnabled(true);
			}
			else {
				okButton.setEnabled(false);
			}
		}
		
	}
	
	private class AssemblyNode extends DefaultMutableTreeNode {
		
		private File f;
		
		public AssemblyNode (File f) {
			super(f.getName());
			this.f = f;
		}
		
		public File file () {
			return f;
		}
		
	}
	
}
