/**
 * Copyright 2006-8 Simon Andrews
 *
 *    This file is part of ChIPMonk.
 *
 *    ChIPMonk is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    ChIPMonk is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with ChIPMonk; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package uk.ac.bbsrc.babraham.ChipMonk.Dialogs;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.filechooser.FileFilter;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableModel;

import com.sun.java.TableSorter;

import uk.ac.bbsrc.babraham.ChipMonk.ChipMonkApplication;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.Probe;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.ProbeList;
import uk.ac.bbsrc.babraham.ChipMonk.Displays.SmallDoubleCellRenderer;
import uk.ac.bbsrc.babraham.ChipMonk.Preferences.ChipMonkPreferences;

public class ProbeListViewer extends JDialog implements MouseListener, ActionListener {
		
	private ChipMonkApplication application;
	private JTable table;
	
	public ProbeListViewer (ProbeList list, ChipMonkApplication application) {
		super(application,list.getListName()+" ("+list.getAllProbes().length+" probes)");

		this.application = application;
		
		Probe [] probes = list.getAllProbes();
		
		getContentPane().setLayout(new BorderLayout());

		String [] headers = new String [] {"Probe","Chr","Pos",list.getValueName()};
		Class [] classes = new Class [] {String.class,String.class,Integer.class,Double.class};
		
		Object [][] rowData = new Object [probes.length][headers.length];
		
		for (int i=0;i<probes.length;i++) {
			rowData[i][0] = probes[i];
			if (probes[i].chromosome() != null) {
				rowData[i][1] = probes[i].chromosome().name();
			}
			else {
				rowData[i][1] = "No chr";				
			}
			rowData[i][2] = new Integer(probes[i].position());
			rowData[i][3] = list.getValueForProbe(probes[i]);
		}

		TableSorter sorter = new TableSorter(new ProbeTableModel(rowData,headers,classes));
		table = new JTable(sorter);
		table.setDefaultRenderer(Double.class, new SmallDoubleCellRenderer());
		table.addMouseListener(this);
		sorter.setTableHeader(table.getTableHeader());

		getContentPane().add(new JScrollPane(table),BorderLayout.CENTER);
		
		JPanel buttonPanel = new JPanel();
		JButton cancelButton = new JButton("Close");
		cancelButton.setActionCommand("close");
		cancelButton.addActionListener(this);
		buttonPanel.add(cancelButton);
		
		JButton saveButton = new JButton("Save");
		saveButton.setActionCommand("save");
		saveButton.addActionListener(this);
		buttonPanel.add(saveButton);
		
		getContentPane().add(buttonPanel,BorderLayout.SOUTH);
		
		setSize(450,300);
		setLocationRelativeTo(application);
		setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
		setVisible(true);
		
	}

	private class ProbeTableModel extends AbstractTableModel {

		private Object [][] data;
		private String [] headers;
		private Class [] classes;
		
		public ProbeTableModel (Object [][] data, String [] headers, Class [] classes) {
			super();
			this.data = data;
			this.headers = headers;
			this.classes = classes;
		}
		
		public int getRowCount() {
			return data.length;
		}

		public int getColumnCount() {
			return data[0].length;
		}

		public Object getValueAt(int r, int c) {
			return data[r][c];
		}
		
		public String getColumnName (int c) {
			return headers[c];
		}
		
		public Class getColumnClass (int c) {
			return classes[c];
		}
		
		public boolean isCellEditable (int r, int c) {
			return false;
		}
	}

	public void mouseClicked(MouseEvent me) {
		//We're only interested in double clicks
		if (me.getClickCount() !=2 ) return;
		// This is only linked from the report JTable
		JTable t = (JTable)me.getSource();
		int r = t.getSelectedRow();
		Probe p = (Probe)t.getValueAt(r,0);
		application.displayChromosome(p.chromosome(),p.position());
		application.chromosomeViewer().setView(p.position()-100,p.position()+100);
	}

	public void mousePressed(MouseEvent arg0) {
	}

	public void mouseReleased(MouseEvent arg0) {
	}

	public void mouseEntered(MouseEvent arg0) {
	}

	public void mouseExited(MouseEvent arg0) {
	}

	public void actionPerformed(ActionEvent ae) {

		if (ae.getActionCommand().equals("close")){
			setVisible(false);
			dispose();
		}
		else if (ae.getActionCommand().equals("save")){
			JFileChooser chooser = new JFileChooser(ChipMonkPreferences.getInstance().getSaveLocation());
			chooser.setMultiSelectionEnabled(false);
			chooser.setFileFilter(new FileFilter() {
			
				public String getDescription() {
					return "Text files";
				}
			
				public boolean accept(File f) {
					if (f.isDirectory() || f.getName().toLowerCase().endsWith(".txt")) {
						return true;
					}
					else {
						return false;
					}
				}
			
			});
			
			int result = chooser.showSaveDialog(this);
			if (result == JFileChooser.CANCEL_OPTION) return;

			File file = chooser.getSelectedFile();
			if (! file.getPath().toLowerCase().endsWith(".txt")) {
				file = new File(file.getPath()+".txt");
			}

			// Check if we're stepping on anyone's toes...
			if (file.exists()) {
				int answer = JOptionPane.showOptionDialog(this,file.getName()+" exists.  Do you want to overwrite the existing file?","Overwrite file?",0,JOptionPane.QUESTION_MESSAGE,null,new String [] {"Overwrite and Save","Cancel"},"Overwrite and Save");

				if (answer > 0) {
					return;
				}
			}

			
			try {
				PrintWriter p = new PrintWriter(new FileWriter(file));
				
				TableModel model = table.getModel();
				
				int rowCount = model.getRowCount();
				int colCount = model.getColumnCount();
				
				// Do the headers first
				StringBuffer b = new StringBuffer();
				for (int c=0;c<colCount;c++) {
					b.append(model.getColumnName(c));
					if (c+1 != colCount) {
						b.append("\t");
					}
				}
				
				p.println(b);
				
				for (int r=0;r<rowCount;r++) {
					b = new StringBuffer();
					for (int c=0;c<colCount;c++) {
						b.append(model.getValueAt(r,c));
						if (c+1 != colCount) {
							b.append("\t");
						}
					}
					p.println(b);
					
				}
				p.close();
			
			}
				
					
			catch (FileNotFoundException e) {
				JOptionPane.showMessageDialog(application,"Can't save probe list: e.getMessage()","Error",JOptionPane.ERROR_MESSAGE);
				e.printStackTrace();
			}
			catch (IOException e) {
				JOptionPane.showMessageDialog(application,"Can't save probe list: e.getMessage()","Error",JOptionPane.ERROR_MESSAGE);
				e.printStackTrace();
			}
			
		}
		
	}
}
