/**
 * Copyright 2006-8 Simon Andrews
 *
 *    This file is part of ChIPMonk.
 *
 *    ChIPMonk is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    ChIPMonk is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with ChIPMonk; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package uk.ac.bbsrc.babraham.ChipMonk.Dialogs.Reports;

import java.awt.BorderLayout;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Enumeration;
import java.util.Vector;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.filechooser.FileFilter;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableModel;

import com.sun.java.TableSorter;

import uk.ac.bbsrc.babraham.ChipMonk.ChipMonkApplication;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.Chromosome;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.DataStore;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.Feature;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.Location;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.Probe;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.ProbeList;
import uk.ac.bbsrc.babraham.ChipMonk.Displays.SmallDoubleCellRenderer;
import uk.ac.bbsrc.babraham.ChipMonk.Preferences.ChipMonkPreferences;


public class ProbeGroupReport extends JDialog implements ActionListener, KeyListener, MouseListener, ItemListener {

	private ChipMonkApplication application;
	private JComboBox annotationPosition;
	private JComboBox annotationType;
	private JComboBox excludes;
	private JComboBox data;
	private JTextField annotationLimit;
	private JTextField probeDistanceLimit;
	private JButton okButton;
	private JDialog reportDialog;
	private TableModel model;
	private ProbeList list;

	public ProbeGroupReport (ChipMonkApplication application) {
		super(application,"Create probe group report...");
		setSize(500,300);
		setLocationRelativeTo(application);
		
		this.application = application;
		this.list = application.dataCollection().probeSet().getActiveProbeList();
		
		getContentPane().setLayout(new BorderLayout());
		
		JPanel choicePanel = new JPanel();
		choicePanel.setLayout(new BoxLayout(choicePanel,BoxLayout.Y_AXIS));
		
		JPanel choicePanel0 = new JPanel();
		JLabel probeListLabel = new JLabel ("Reporting on probes in '"+application.dataCollection().probeSet().getActiveProbeListName()+"' ("+application.dataCollection().probeSet().getActiveProbes().length+" probes)",JLabel.CENTER);
		probeListLabel.setFont(new Font("Default",Font.BOLD,12));
		choicePanel0.add(probeListLabel);
		choicePanel.add(choicePanel0);
		
		JPanel choicePanel1 = new JPanel();
		
		choicePanel1.add(new JLabel("Annotate with "));
		annotationPosition = new JComboBox(new String [] {"all overlapping","all enclosed","closest", "surrounding", "upstream", "downstream", "surrounding or upstream", "surrounding or downstream"});
		annotationPosition.addItemListener(this);
		choicePanel1.add(annotationPosition);
		annotationType = new JComboBox(application.genome().listAvailableFeatureTypes());
		choicePanel1.add(annotationType);
		choicePanel.add(choicePanel1);
		
		
		JPanel choicePanel2 = new JPanel();
		choicePanel2.add(new JLabel("Annotation distance cutoff "));
		annotationLimit = new JTextField("2000",7);
		annotationLimit.addKeyListener(this);
		annotationLimit.setEnabled(false);
		choicePanel2.add(annotationLimit);
		choicePanel2.add(new JLabel("bp"));
		choicePanel.add(choicePanel2);
		
		JPanel choicePanel3 = new JPanel();
		excludes = new JComboBox(new String [] {"Include","Exclude"});
		choicePanel3.add(excludes);
		choicePanel3.add(new JLabel(" unannotated probes"));
		choicePanel.add(choicePanel3);
		
		getContentPane().add(choicePanel,BorderLayout.CENTER);

		JPanel choicePanel4 = new JPanel();
		data = new JComboBox(new String [] {"Include","Don't include"});
		choicePanel4.add(data);
		choicePanel4.add(new JLabel(" data for currently visible stores"));
		choicePanel.add(choicePanel4);
		
		JPanel choicePanel5 = new JPanel();
		choicePanel5.add(new JLabel("Group probes within "));
		probeDistanceLimit = new JTextField("5000",7);
		probeDistanceLimit.addKeyListener(this);
		choicePanel5.add(probeDistanceLimit);
		choicePanel5.add(new JLabel(" bp"));
		choicePanel.add(choicePanel5);
		
		getContentPane().add(choicePanel,BorderLayout.CENTER);

		
		JPanel buttonPanel = new JPanel();
		JButton cancelButton = new JButton("Cancel");
		cancelButton.setActionCommand("cancel");
		cancelButton.addActionListener(this);
		buttonPanel.add(cancelButton);
		
		okButton = new JButton("OK");
		okButton.setActionCommand("ok");
		okButton.addActionListener(this);
		buttonPanel.add(okButton);
		
		getContentPane().add(buttonPanel,BorderLayout.SOUTH);
		
		setVisible(true);
	}

	private void makeReport () {
		
		String annotationTypeValue = (String)annotationType.getSelectedItem();
		int distanceLimit = 0;
		
		// Check what to do with unannotated probes
		boolean includeAll = true;
		if (((String)excludes.getSelectedItem()).equals("Exclude")) {
			includeAll = false;
		}
		
		String annotationPositionValue = (String)annotationPosition.getSelectedItem();
		// We're going to set up a set of booleans which tell us which kinds
		// of relationships we're allowed to look for later.
		boolean annotateIndividualProbes = true;
		boolean justEnclosed = false;
		boolean surrounding = true;
		boolean upstream = true;
		boolean downstream = true;
		
		if (annotationPositionValue.equals("all overlapping")) {
			annotateIndividualProbes = false;
			justEnclosed = false;
		}
		else if (annotationPositionValue.equals("all enclosed")) {
			annotateIndividualProbes = false;
			justEnclosed = true;
		}
		else if (annotationPositionValue.equals("surrounding")) {
			upstream = false;
			downstream = false;
		}
		else if (annotationPositionValue.equals("surrounding or upstream")) {
			downstream = false;
		}
		else if (annotationPositionValue.equals("surrounding or downstream")) {
			upstream = false;
		}
		else if (annotationPositionValue.equals("upstream")) {
			surrounding = false;
			downstream = false;
		}
		else if (annotationPositionValue.equals("downstream")) {
			surrounding = false;
			upstream = false;
		}
		else if (annotationPositionValue.equals("closest")) {
			// Leave things as they are!
		}
		else {
			System.err.println("Didn't recognise position value '"+annotationPositionValue+"'");
		}
	
		// We only need to worry about a distance cutoff if we're not using
		// surrounding.
		if (!annotationTypeValue.equals("surrounding")) {
			if (annotationLimit.getText().length()> 0) {
				distanceLimit = Integer.parseInt(annotationLimit.getText());
			}
		}
		
		// Find out how closely to group probes
		int groupDistance = Integer.parseInt(probeDistanceLimit.getText());
				
		Vector annotations = new Vector();
		
		// Since we're going to be making the annotations on the
		// basis of position we should go through all probes one
		// chromosome at a time.
		
		Chromosome [] chrs = application.genome().getAllChromosomes();
		
		for (int c=0;c<chrs.length;c++) {
			Probe [] probes = application.dataCollection().probeSet().getActiveProbesForChromosome(chrs[c]);
			Feature [] features = chrs[c].getFeaturesForType(annotationTypeValue);
			
			ProbeGroupAnnotation group = new ProbeGroupAnnotation();
			int lastPosition = -1;
			
			// We can now step through the probes looking for the best feature match
			for (int p=0;p<probes.length;p++) {
				
//				System.err.println("Looking at probe at pos "+probes[p].position()+" last was "+lastPosition);
				
				// Check if we're still within the same probe group as before
				
				if (lastPosition == -1 || probes[p].position()-groupDistance <= lastPosition) {
					// We're in the same group
					group.addProbe(probes[p]);
					lastPosition = probes[p].position();
//					System.err.println("Adding to existing group");
				}
				else {
					// We're at the end of a group so create a new one
//					System.err.println("End of group");
					
					// Since we have a completed group we may need to do the
					// group annotations
					if (! annotateIndividualProbes) {
						annotateGroup(group,justEnclosed,annotationTypeValue);
					}
					if(includeAll || group.featureNames().length()>0) {
//						System.err.println("Adding group to keepers");
						annotations.add(group);
					}
					else {
//						System.err.println("Not adding this group");
					}
					
					// Create a new group and add the current probe to start it off.
					group = new ProbeGroupAnnotation();
					group.addProbe(probes[p]);
					lastPosition = probes[p].position();
				}

				if (! annotateIndividualProbes) continue;
				
				Feature bestFeature = null;
				int closestDistance = 0;
				
				for (int f=0;f<features.length;f++) {
					
					if (surrounding) {
						// Quickest check is whether a probe is inside a feature
					
						if (probes[p].position() >= features[f].location().start() && probes[p].position() <= features[f].location().end()) {
							bestFeature = features[f];
							closestDistance = 0;
						
							// Once we've found an overlapping feature we quit.
							break;
						}
					}
					
					if (downstream) {
						// Check if the feature is downstream
						// Get the distance to the start
						int d=0;
						if (features[f].location().strand() == Location.FORWARD) {
							d = features[f].location().start() - probes[p].position();
						}
						else {
							d = probes[p].position() - features[f].location().end();
						}
						
						if (d >= 0) {
							// We're the right side of the feature

							if (d > distanceLimit || (bestFeature != null && d > closestDistance)) {
								continue;
							}

							// If we make it here this is our best candidate feature so far
							bestFeature = features[f];
							closestDistance = d;
							continue;
						}
					}

					if (upstream) {
						// Check if the feature is upstream
						// Get the distance to the start
						int d=0;
						if (features[f].location().strand() == Location.FORWARD) {
							d = probes[p].position() - features[f].location().end();
						}
						else {
							d = features[f].location().start() - probes[p].position();
						}
						
						if (d >= 0) {
							// We're the right side of the feature

							if (d > distanceLimit || (bestFeature != null && d > closestDistance)) {
								continue;
							}

							// If we make it here this is our best candidate feature so far
							bestFeature = features[f];
							closestDistance = d;
							continue;
						}
					}
				
				}
				if (bestFeature != null) {
					group.addFeature(bestFeature);
				}
			}
			
			// We've reached the end of the chromosome
			
			// We still need to check whether to add the current group.
			// and do any necessary annotation.
			if (group.numberOfProbes()>0) {
				// We need to do the annotation for this last group (if we have to)
				if (! annotateIndividualProbes) {
					annotateGroup(group,justEnclosed,annotationTypeValue);
				}
				if(includeAll || group.featureNames().length()>0) {
//					System.err.println("Adding group to keepers");
					annotations.add(group);
				}
			}
		}
				
		
		reportDialog = new JDialog(application,"Report for '"+application.dataCollection().probeSet().getActiveProbeListName()+"'");
		
		DataStore [] stores = new DataStore[0];
		
		if (((String)data.getSelectedItem()).equals("Include")) {
			stores = application.drawnDataStores();
		}
		
		model = new TableSorter(new AnnotationTableModel((ProbeGroupAnnotation [])annotations.toArray(new ProbeGroupAnnotation[0]),stores));
		JTable table = new JTable(model);
		table.setDefaultRenderer(Double.class, new SmallDoubleCellRenderer());
		table.addMouseListener(this);
		((TableSorter)model).setTableHeader(table.getTableHeader());

		
		reportDialog.getContentPane().setLayout(new BorderLayout());
		
		reportDialog.getContentPane().add(new JScrollPane(table),BorderLayout.CENTER);
		
		JPanel buttonPanel = new JPanel();
		JButton cancelButton = new JButton("Close");
		cancelButton.setActionCommand("close");
		cancelButton.addActionListener(this);
		buttonPanel.add(cancelButton);
		
		JButton saveButton = new JButton("Save");
		saveButton.setActionCommand("save");
		saveButton.addActionListener(this);
		buttonPanel.add(saveButton);
		
		reportDialog.getContentPane().add(buttonPanel,BorderLayout.SOUTH);

		
		reportDialog.setSize(600,400);
		reportDialog.setLocationRelativeTo(application);
		reportDialog.setVisible(true);
		
		setVisible(false);
		dispose();
		
	}
	
	private void annotateGroup (ProbeGroupAnnotation g, boolean justEnclosed, String featureType) {

		if (g.chromosome() == null) return;
		
		Feature [] features = g.chromosome().getFeaturesForType(featureType);
		
		for (int f=0;f<features.length;f++) {
			
			if (justEnclosed) {
				if (features[f].location().start()>=g.start() && features[f].location().end()<=g.end()) {
					g.addFeature(features[f]);
				}
			}
			else {
				if (features[f].location().start()<g.end() && features[f].location().end()>g.start()) {
					g.addFeature(features[f]);
				}				
			}
		}

	}
	
	private void checkOK () {
		// Check to see if enough information has been added to allow us to
		// enable the OK button.
		
		if (probeDistanceLimit.getText().length() == 0) {
			okButton.setEnabled(false);
			return;
		}
		
		// If we get here then we're good to go
		okButton.setEnabled(true);
		
	}
	
	public void actionPerformed(ActionEvent ae) {
		if (ae.getActionCommand().equals("cancel")) {
			setVisible(false);
			dispose();
		}
		else if (ae.getActionCommand().equals("ok")) {
			makeReport();
		}
		// These two actions come from the opened report
		else if (ae.getActionCommand().equals("close")){
			reportDialog.setVisible(false);
			reportDialog.dispose();
		}
		else if (ae.getActionCommand().equals("save")){
			JFileChooser chooser = new JFileChooser(ChipMonkPreferences.getInstance().getSaveLocation());
			chooser.setMultiSelectionEnabled(false);
			chooser.setFileFilter(new FileFilter() {
			
				public String getDescription() {
					return "Text files";
				}
			
				public boolean accept(File f) {
					if (f.isDirectory() || f.getName().toLowerCase().endsWith(".txt")) {
						return true;
					}
					else {
						return false;
					}
				}
			
			});
			
			int result = chooser.showSaveDialog(this);
			if (result == JFileChooser.CANCEL_OPTION) return;

			File file = chooser.getSelectedFile();
			if (! file.getPath().toLowerCase().endsWith(".txt")) {
				file = new File(file.getPath()+".txt");
			}

			// Check if we're stepping on anyone's toes...
			if (file.exists()) {
				int answer = JOptionPane.showOptionDialog(this,file.getName()+" exists.  Do you want to overwrite the existing file?","Overwrite file?",0,JOptionPane.QUESTION_MESSAGE,null,new String [] {"Overwrite and Save","Cancel"},"Overwrite and Save");

				if (answer > 0) {
					return;
				}
			}

			try {
				PrintWriter p = new PrintWriter(new FileWriter(file));
				
				int rowCount = model.getRowCount();
				int colCount = model.getColumnCount();
				
				// Do the headers first
				StringBuffer b = new StringBuffer();
				for (int c=0;c<colCount;c++) {
					b.append(model.getColumnName(c));
					if (c+1 != colCount) {
						b.append("\t");
					}
				}
				
				p.println(b);
				
				for (int r=0;r<rowCount;r++) {
					b = new StringBuffer();
					for (int c=0;c<colCount;c++) {
						b.append(model.getValueAt(r,c));
						if (c+1 != colCount) {
							b.append("\t");
						}
					}
					p.println(b);
					
				}
				p.close();
			
			}
				
					
			catch (FileNotFoundException e) {
				JOptionPane.showMessageDialog(application,"Can't save report: e.getMessage()","Error",JOptionPane.ERROR_MESSAGE);
				e.printStackTrace();
			}
			catch (IOException e) {
				JOptionPane.showMessageDialog(application,"Can't save report: e.getMessage()","Error",JOptionPane.ERROR_MESSAGE);
				e.printStackTrace();
			}
			
		}
		
	}

	public void keyTyped(KeyEvent arg0) {
	}

	public void keyPressed(KeyEvent ke) {

	}

	public void keyReleased(KeyEvent ke) {
		JTextField f = (JTextField)ke.getSource();
		
		char [] c = f.getText().toCharArray();
		
		StringBuffer b = new StringBuffer();
		for (int i=0;i<c.length;i++) {
			
			if (Character.isDigit(c[i])) {
				b.append(c[i]);
				continue;
			}

			f.setText(b.toString());
			break;
		}
		
		
		checkOK();
	}
	
	private class ProbeGroupAnnotation {
		
		public Vector probes;
		public Vector features;
		
		public ProbeGroupAnnotation () {
			probes = new Vector();
			features = new Vector();
		}
		
		public void addProbe (Probe p) {
			probes.add(p);
		}
		public void addFeature (Feature f) {
			if (! features.contains(f))
				features.add(f);
		}
		public int start () {
			Enumeration e = probes.elements();
			int start = 0;
			while (e.hasMoreElements()) {
				Probe p = (Probe)e.nextElement();
				if (start == 0) {
					start = p.position();
				}
				
				if (p.position() < start)
					start = p.position();
			}
			
			return start;
		}
		
		public Probe [] probes () {
			return (Probe [])probes.toArray(new Probe[0]);
		}
		
		public int end () {
			Enumeration e = probes.elements();
			int end = 0;
			while (e.hasMoreElements()) {
				Probe p = (Probe)e.nextElement();
				if (end == 0) {
					end = p.position();
				}
				
				if (p.position() > end)
					end = p.position();
			}
			
			return end;
		}
		
		public Double getAnnotationValue () {
			double value = 0;
			int count = 0;
			
			Enumeration e = probes.elements();
			while (e.hasMoreElements()) {
				Probe p = (Probe)e.nextElement();
				Double d = list.getValueForProbe(p);
				if (d != null) {
					value += d.doubleValue();
					++count;
				}
			}
			if (count > 0) {
				return new Double(value/count);
			}
			else {
				return new Double(Double.NaN);
			}
		}
		
		public Chromosome chromosome () {
			if (probes.size()>0) {
				return ((Probe)probes.elementAt(0)).chromosome();
			}
			else {
				return null;
			}
		}
		
		public int numberOfProbes () {
			return probes.size();
		}
		
		public String featureNames () {
			StringBuffer b = new StringBuffer();
			Enumeration e = features.elements();
			while (e.hasMoreElements()) {
				b.append(((Feature)e.nextElement()).name());
				b.append(" ");
			}
			return b.toString();
		}
		
		public String featureDescriptions () {
			StringBuffer b = new StringBuffer();
			Enumeration e = features.elements();
			while (e.hasMoreElements()) {
				b.append(((Feature)e.nextElement()).description());
				b.append(" ");
			}
			return b.toString();
			
		}
		
	}
	
	private class AnnotationTableModel extends AbstractTableModel  {
		
		private ProbeGroupAnnotation [] data;

		private DataStore [] stores;
		
		public AnnotationTableModel (ProbeGroupAnnotation [] data, DataStore [] stores) {
//			System.err.println("Created table with "+data.length+" rows");
			this.data = data;
			this.stores = stores;
		}

		public int getRowCount() {
			return data.length;
		}

		public int getColumnCount() {
			return 7+stores.length;
		}

		public String getColumnName (int c) {
			switch (c) {
			case 0: return "Chr";
			case 1: return "Start";
			case 2: return "End";
			case 3: if (list != null)return "Average "+list.getValueName(); else return "No value";
			case 4: return "No. Probes";
			case 5: return "Features";
			case 6: return "Descriptions";
			default: return stores[c-7].name();
			}
		}

		public Class getColumnClass (int c) {
			switch (c) {
			case 0: return String.class;
			case 1: return Integer.class;
			case 2: return Integer.class;
			case 3: return Double.class;
			case 4: return Integer.class;
			case 5: return String.class;
			case 6: return String.class;
			default: return Double.class;
			}
		}

		public ProbeGroupAnnotation getGroupAnnotationForRow (int r) {
			return data[r];
		}
		
		public Object getValueAt(int r, int c) {
			switch (c) {
			case 0 :
				return data[r].chromosome();
			
			case 1:
				return new Integer(data[r].start());
				
			case 2:
				return new Integer(data[r].end());
				
			case 3:
				return data[r].getAnnotationValue();

			case 4:
				return new Integer(data[r].numberOfProbes());
				
			case 5:
				return data[r].featureNames();
				
			case 6:
				return data[r].featureDescriptions();
				
			default:
				Probe [] theseProbes = data[r].probes();
				double ratios = 0;
				for (int i=0;i<theseProbes.length;i++) {
					ratios += stores[c-7].getRatioForProbe(theseProbes[i]);
				}
				return new Double(ratios/theseProbes.length);
			}
		}
		
	}

	public void mouseClicked(MouseEvent me) {
		//We're only interested in double clicks
		if (me.getClickCount() !=2 ) return;
		// This is only linked from the report JTable
		JTable t = (JTable)me.getSource();
		int r = t.getSelectedRow();
		application.displayChromosome((Chromosome)t.getValueAt(r,0),((Integer)t.getValueAt(r,1)).intValue());
		application.chromosomeViewer().setView(((Integer)t.getValueAt(r,1)).intValue()-100,((Integer)t.getValueAt(r,2)).intValue()+100);
	}

	public void mousePressed(MouseEvent arg0) {
	}
	public void mouseReleased(MouseEvent arg0) {
	}
	public void mouseEntered(MouseEvent arg0) {
	}
	public void mouseExited(MouseEvent arg0) {
	}

	public void itemStateChanged(ItemEvent arg0) {
		String a = (String)annotationPosition.getSelectedItem();
		
		if (a.equals("all overlapping") || a.equals("all enclosed") || a.equals("surrounding")) {
			annotationLimit.setEnabled(false);
		}
		else {
			annotationLimit.setEnabled(true);
		}
	}
	
}
