/**
 * Copyright 2006-8 Simon Andrews
 *
 *    This file is part of ChIPMonk.
 *
 *    ChIPMonk is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    ChIPMonk is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with ChIPMonk; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package uk.ac.bbsrc.babraham.ChipMonk.Network;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.net.URL;
import java.util.Enumeration;
import java.util.Vector;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

import uk.ac.bbsrc.babraham.ChipMonk.ChipMonkException;
import uk.ac.bbsrc.babraham.ChipMonk.Preferences.ChipMonkPreferences;

public class GenomeDownloader implements Runnable {

	private Vector listeners = new Vector();
	private ChipMonkPreferences prefs = ChipMonkPreferences.getInstance();
	private String species;
	private String assembly;
	private int size;
	
	
	public void downloadGenome (String species, String assembly, int size) {
		
		this.species = species;
		this.assembly = assembly;
		this.size = size;
		Thread t = new Thread(this);
		t.start();
	}
	
	public void addGenomeDownloaderListener (GenomeDownloaderListener gdl) {
		listeners.add(gdl);
	}
	
	public void removeGeneomeDownloaderLister (GenomeDownloaderListener gdl) {
		listeners.remove(gdl);
	}
		
	public void run() {
		// First we need to download the file from the repository
		try {
//			System.out.println("Downloading "+prefs.getGenomeDownloadLocation()+species+"/"+assembly+".zip");
			URL url = new URL((new String(prefs.getGenomeDownloadLocation()+species+"/"+assembly+".zip")).replaceAll(" ","%20"));
			
			DataInputStream d = new DataInputStream(new BufferedInputStream(url.openStream()));
//			System.out.println("Output file is "+prefs.getGenomeBase()+"/"+assembly+".zip");
			File f = new File(prefs.getGenomeBase()+"/"+assembly+".zip");
			DataOutputStream o = new DataOutputStream(new BufferedOutputStream(new FileOutputStream(f)));
			byte [] b = new byte [1024];
			int totalBytes = 0;
			int i;
			while ((i=d.read(b))>0){
//				System.out.println("Read "+totalBytes+" bytes");
				o.write(b,0,i);
				totalBytes += i;
				Enumeration en = listeners.elements();
				
				while (en.hasMoreElements()) {
					((GenomeDownloaderListener)en.nextElement()).downloadUpdateProgress(totalBytes,size,"Downloaded "+totalBytes/1048576+"Mb");
				}
			}

			d.close();
			o.close();
			
			File outFile = new File(prefs.getGenomeBase()+"/"+species+"/"+assembly);
			outFile.mkdirs();
			// Now we can uncompress the downloaded file to create the genome
			ZipFile zipFile = new ZipFile(f);
			Enumeration e = zipFile.entries();
			int count = 0;
			while (e.hasMoreElements()) {
				count++;
				Enumeration en = listeners.elements();
				
				while (en.hasMoreElements()) {
					((GenomeDownloaderListener)en.nextElement()).downloadUpdateProgress(count,zipFile.size(),"Unzipped "+count+" files");
				}
				
				ZipEntry ze = (ZipEntry)e.nextElement();

				if (ze.isDirectory()) continue;
//				System.out.println("Found entry "+ze.getName()+" with size "+ze.getSize());
				BufferedOutputStream out;
				try {	
					out  = new BufferedOutputStream(new FileOutputStream(prefs.getGenomeBase()+"/"+ze.getName()));
				}
				catch (FileNotFoundException fnfe) {
					throw new ChipMonkException("Couldn't write into the genomes directory.  Please check your file permissions or change your genomes folder.");
				}
				BufferedInputStream in = new BufferedInputStream(zipFile.getInputStream(ze));
				while ((i=in.read(b))>0) {
					out.write(b,0,i);
				}
				in.close();
				out.close();
			}
			// Finally, get rid of the zip file so we don't waste disk space.
			zipFile.close();
			f.delete();
			
		} 
		catch (Exception ex) {
			ex.printStackTrace();
			Enumeration en = listeners.elements();
			
			if (ex instanceof FileNotFoundException) {
				ex = new ChipMonkException("Couldn't find the '"+species+" "+assembly+"' genome on the download server. Check your network preferences.");
			}
			
			while (en.hasMoreElements()) {
				((GenomeDownloaderListener)en.nextElement()).downloadExceptionReceived(ex);
			}
			return;
		}

		// Tell everyone we're finished
		Enumeration en = listeners.elements();
		
		while (en.hasMoreElements()) {
			((GenomeDownloaderListener)en.nextElement()).downloadProcessingFinished();
		}
		
	}
	
}
