/**
 * Copyright 2006-8 Simon Andrews
 *
 *    This file is part of ChIPMonk.
 *
 *    ChIPMonk is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    ChIPMonk is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with ChIPMonk; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package uk.ac.bbsrc.babraham.ChipMonk.Normalisation;

import java.util.Arrays;
import java.util.Enumeration;
import java.util.Vector;

import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.DataCollection;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.DataSet;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.Probe;

public class PerProbeNormalisation implements Normalisation, Runnable {

	private Vector listeners = new Vector();
	private DataCollection data;
	
	public void normalise(DataCollection d) {
		data = d;
		Thread t = new Thread(this);
		t.start();
	}

	public void addNormalisationProgressListener(NormalisationProgressListener l) {
		listeners.add(l);
	}

	public boolean isSuitable(DataCollection d) {
		return true;
	}

	public void run() {
		Probe [] probes = data.probeSet().getAllProbes();
		DataSet [] sets = data.getAllDataSets();
		
		for (int p=0;p<probes.length;p++) {
			if (p%1000 == 0) {
				Enumeration e = listeners.elements();
				while (e.hasMoreElements()) {
					NormalisationProgressListener l = (NormalisationProgressListener)e.nextElement();
					l.updateNormalisationProgress("Per probe Normalisation "+p+" probes done");
				}
				try {
					Thread.sleep(20);
				} catch (InterruptedException e1) {
				}
			}
			
			double [] signals = new double [sets.length];
			
			for (int s=0;s<sets.length;s++) {
				signals[s] = sets[s].getSignalValueForProbe(probes[p]);
			}

			// Now sort the signals
			Arrays.sort(signals);
			
			// We need the median value
			double median = 0;
			if (signals.length % 2 == 0) {
				// We need to take the middle value
				median = new Double(signals[signals.length/2]).doubleValue();
			}
			else {
				// We need to average the middle two values
				median = (signals[signals.length/2]+signals[(signals.length/2)+1])/2;
			}
			
			//Just for debugging
//			System.out.println("Per probe Normalisation for "+probes[p].name());
//			System.out.println("Original Signal Values:");
//			for (int x=0;x<signals.length;x++)
//				System.out.println(signals[x]);
//			System.out.println("Median is "+median);
//			
//			//Just for debugging
//			System.out.println("Modified Signal Values:");
			
			// We now need to divide everything by that median value
			for (int s=0;s<sets.length;s++) {
				sets[s].setSignalForProbe(probes[p],sets[s].getSignalValueForProbe(probes[p])/median);
//				System.out.println(sets[s].getSignalValueForProbe(probes[p]));
			}


			
			// Do the same thing again for the controls (but reuse the same array
			signals = new double[sets.length];

			for (int s=0;s<sets.length;s++) {
				signals[s] = sets[s].getControlValueForProbe(probes[p]);
			}

			// Now sort the signals
			Arrays.sort(signals);
			
			// We need the median value
			median = 0;
			if (signals.length % 2 == 0) {
				// We need to take the middle value
				median = new Double(signals[signals.length/2]).doubleValue();
			}
			else {
				// We need to average the middle two values
				median = (signals[signals.length/2]+signals[(signals.length/2)+1])/2;
			}

//			//Just for debugging
//			System.out.println("Original Control Values:");
//			for (int x=0;x<signals.length;x++)
//				System.out.println(signals[x]);
//			System.out.println("Median is "+median);
//
//			//Just for debugging
//			System.out.println("Modified Control Values:");			
			
			// We now need to divide everything by that median value
			for (int s=0;s<sets.length;s++) {
				sets[s].setControlForProbe(probes[p],sets[s].getControlValueForProbe(probes[p])/median);
//				System.out.println(sets[s].getControlValueForProbe(probes[p]));
			}
			
		}

		Enumeration e = listeners.elements();
		while (e.hasMoreElements()) {
			((NormalisationProgressListener)e.nextElement()).normalisationComplete();
		}
	}
	
	public String toString () {
		return shortDescription();
	}

	public String shortDescription() {
		return "Per Probe Normalisation";
	}

	public String longDescription() {
		return "<html>Normalises all of the data for a probe so that the average ratio across all samples is 1<br>Shows the varation between groups</html>";
	}


}
